import argparse
from pixiu.pxtester import PXTester

def str2bool(v):
    if isinstance(v, bool):
       return v
    if v.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    else:
        raise argparse.ArgumentTypeError('Boolean value expected.')

def main(*args, **kwargs):
    parser = argparse.ArgumentParser()
    parser.add_argument('-c', '--testconfig', type=str, required=True, help='Test config path')
    parser.add_argument('-n', '--testname', type=str, required=True, help='Test name')
    parser.add_argument('-s', '--scriptpath', type=str, required=True, help='Script path')
    parser.add_argument('-o', '--logpath', type=str, help='Log path')
    parser.add_argument('-p', '--printlogtype', nargs='+', default=['ea', 'report'], required=False,
                        help='Print log type,  account order ea report')
    args = parser.parse_args()
    pxt = PXTester(test_config_path=args.testconfig, test_name=args.testname, script_path=args.scriptpath,
                   log_path=args.logpath, print_log_type=args.printlogtype)
    pxt.execute("", sync=True)


if __name__ == '__main__':
    main()





import time
import traceback
from RestrictedPython import (compile_restricted, safe_globals, utility_builtins,)
from RestrictedPython.PrintCollector import PrintCollector



import logging
log = logging.getLogger(__name__)

#---------------------------------------------------------------------------------------------------------------------
# EA Classes
#---------------------------------------------------------------------------------------------------------------------
class EABase():
    """"""
    def __init__(self, params):
        self.language = 'en'
        self.symbol = None
        self.byte_code = None
        self.script = None
        self.script_path = None
        self.script_settings = None
        self.last_exec_time = 0
        self.safe_globals = safe_globals.copy()
        self.global_values = params.get('global_values', {})
        self.loc = {}
        self.df_columns = ('t', 's', 'o', 'h', 'l', 'c', 'v', 'a', 'b')
        self.ea_log_callback = params.get('ea_log_callback', None)
        #

    def get_print_factory(self, _getattr_=None):
        """"""
        if self.print_collection is None:
            self.print_collection = PrintCollector(_getattr_)
        return self.print_collection

    def get_param(self, name, default=None):
        try:
            if self.script_settings and isinstance(self.script_settings, dict):
                params = self.script_settings.get("params", {})
                if name in params.keys():
                    params_value = params[name]
                    if params_value is None or not isinstance(params_value, dict):
                        return params_value
                    if "config" in params_value.keys():
                        return params_value.get("value", default)
                    else:
                        return params_value
        except:
            traceback.print_exc()
        return default
    #
    # #
    def get_symbol_properties(self, symbol):
        ''''''
        raise NotImplemented

    @staticmethod
    def compile(script, filename='<inline>'):
        '''编译脚本'''
        return compile_restricted(script, filename, 'exec')

    def do_tick(self,):
        if not self.byte_code:
            if self.script_path is None:
                self.byte_code = EABase.compile(self.script)
            else:
                self.byte_code = EABase.compile(open(self.script_path).read(), self.script_path)

        self.order_log = []
        stime = time.time()
        exec(self.byte_code, self.safe_globals)
        self.last_exec_time = time.time() - stime
        #




'''
PIXIU API
'''
from typing import NewType, TypeVar, Generic
import abc
from datetime import datetime
from pixiu.api.defines import (TimeFrame, OrderType, OrderCommand)
from .order import Order

class IndicatiorID():
    MA = 1000
    AD = 2000
    ADX = 3000
    ATR = 4000
    BANDS = 5000
    CCI = 6000
    CHAIKIN = 7000
    DEMA = 8000
    MOMENTUM = 9000
    MFI = 10000
    MACD = 11000
    OBV = 12000
    SAR = 13000
    RSI = 14000
    STDDEV = 15000
    STOCHASTIC = 16000
    TEMA = 17000
    WPR = 18000

class OrderScope():
    EA = 0
    EA_VERSION = 200
    ACCOUNT = 300

ErrorID = NewType('ErrorID', int)
Result = NewType('Result', dict)
UID = NewType('UID', str)
OrderUID = NewType('OrderUID', UID)
OrderResult = NewType('OrderResult', Result)
CommandResult = NewType('CommandResult', Result)


class APIStub(abc.ABC):
    __global_defines__ = dict(
        TimeFrame=TimeFrame,
        OrderType=OrderType,
        OrderCommand=OrderCommand,
        Order=Order,
    )

    def __init__(self):
        pass

    @abc.abstractmethod
    def DefaultTimeFrame(self):
        '''
        Returns the default time frame.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def Buy(self, volume: float, type=OrderType.MARKET, price=None, stop_loss=None, take_profit=None,
            magic_number=None, symbol=None, slippage=None, arrow_color=None, expiration=None) -> (ErrorID, OrderResult):
        '''
        Open a buy order.

                Parameters:
                        volume (float): Number of lots.
                        type (OrderType): Order type.
                        price (float): Order price. If price is None, price = Ask().
                        stop_loss (float): Stop loss price.
                        take_profit (float): Take profit price.
                        magic_number (float): Order magic number.
                        symbol (float): Symbol for trading.
                        slippage (float): Maximum price slippage for trading.
                        arrow_color (float): Color of the opening arrow on the MT4/5 chart.
                        expiration (float): Order expiration time (for pending order only)
                Returns:
                        ErrorID: If 0 success.
                        OrderResult: The order result.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def Sell(self, volume: float, type=OrderType.MARKET, price=None, stop_loss=None, take_profit=None,
             magic_number=None, symbol=None, slippage=None, arrow_color=None, expiration=None) -> (ErrorID, OrderResult):
        '''
        Open a sell order.

                Parameters:
                        volume (float): Number of lots.
                        type (OrderType): Order type.
                        price (float): Order price. If price is None, price = Bid().
                        stop_loss (float): Stop loss price.
                        take_profit (float): Take profit price.
                        magic_number (float): Order magic number.
                        symbol (float): Symbol for trading.
                        slippage (float): Maximum price slippage for trading.
                        arrow_color (float): Color of the opening arrow on the MT4/5 chart.
                        expiration (float): Order expiration time (for pending order only)
                Returns:
                        ErrorID: If 0 success.
                        OrderResult: The order result.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def ModifyOrder(self, uid, price=None, stop_loss=None, take_profit=None,
                     arrow_color=None, expiration=None) -> (ErrorID, OrderResult):
        '''
        Modify a order.

                Parameters:
                        uid : The order UID.
                        price (float): New open price. (for pending order only)
                        stop_loss (float): New stop loss price.
                        take_profit (float): New take profit price.
                        arrow_color (float): New color of the opening arrow on the MT4/5 chart.
                        expiration (float): New order expiration time (for pending order only)
                Returns:
                        ErrorID: If 0 success.
                        OrderResult: The order result.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    # def CloseOrder(self, uid, price, volume: float, slippage=None, arrow_color=None) -> (ErrorID, OrderResult):
    def CloseOrder(self, uid, volume=None, price=None, slippage=None, arrow_color=None) -> (ErrorID, OrderResult):
        '''
        Close a order.

                Parameters:
                        uid : The order UID.
                        volume (float): Number of lots. If volume is None or zero, volume = order.volume
                        price (float): Close price. If price is None or zero, price = OrderClosePrice(MT4/MT5)
                        slippage (float): Maximum price slippage for trading.
                        arrow_color (float): New color of the opening arrow on the MT4/5 chart.
                Returns:
                        ErrorID: If 0 success.
                        OrderResult: The order result.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def GetSymbolData(self, symbol: str, timeframe: str, size: int):
        '''
        Get a symbol data.

                Parameters:
                        symbol (str): Symbol name.
                        timeframe (str): Timeframe. See the TimeFrame defining.
                        size (int): Maximum size.

                Returns:
                        Symbol data
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def StopTester(self, code: int = 0, message: str = None):
        '''
        Stop the EA tester. (For tester only.)

                Parameters:
                        code (int): Error code.
                        message (str): Error message.

                Returns:
                        None
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def GetSymbol(self, symbol=None):
        '''
        Returns the symbol properties.

                Parameters:
                        symbol (int): The symbol name.

                Returns:
                        The symbol properties
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def GetAccount(self):
        '''
        Returns the account data.

                Returns:
                        The account data.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def AccountEquity(self, ):
        '''
        Returns the equity.

                Returns:
                        The account equity.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def AccountFreeMargin(self, ):
        '''
        Returns the free margin.

                Returns:
                        The free margin.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def SymbolInfo(self, item, symbol=None, default=None):
        '''
        Returns the symbol information.

                Parameters:
                        item (str): The symbol item name.
                        symbol (str): The symbol name.
                        default (): The default value.

                Returns:
                        The symbol information.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def GetParam(self, name, default=None):
        '''
        Returns the EA parameter value.

                Parameters:
                        name (): The EA parameter name.
                        default (int): The EA parameter default value.

                Returns:
                        The EA parameter value.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def OrderStats(self, order_uids):
        '''
        Returns the order statistics.

                Parameters:
                        order_uids : the list of order uids.

                Returns:
                        The order statistics.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def GetOrder(self, order_uid: OrderUID):
        '''
        Returns the order object.

                Parameters:
                        order_uid (): The order uid.

                Returns:
                        The order object.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def Close(self, shift=0) -> float:
        '''
        Returns Close price value for the default symbol with default timeframe and shift.

                Parameters:
                        shift (int): Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Close price.
        '''
        raise NotImplementedError
    #
    @abc.abstractmethod
    def Open(self, shift=0) -> float:
        '''
        Returns Open price value for the default symbol with default timeframe and shift.

                Parameters:
                        shift (int): Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Open price.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def High(self, shift=0) -> float:
        '''
        Returns High price value for the default symbol with default timeframe and shift.

                Parameters:
                        shift (int): Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        High price.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def Low(self, shift=0) -> float:
        '''
        Returns Low price value for the default symbol with default timeframe and shift.

                Parameters:
                        shift (int): Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Low price.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def Ask(self, shift=0) -> float:
        '''
        Returns Ask price value for the default symbol with default timeframe and shift.

                Parameters:
                        shift (int): Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Ask price.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def Bid(self, shift=0) -> float:
        '''
        Returns Bid price value for the default symbol with default timeframe and shift.

                Parameters:
                        shift (int): Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Bid price.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def Time(self, shift=0) -> datetime:
        '''
        Returns time value for the default symbol with default timeframe and shift.

                Parameters:
                        shift (int): Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Time.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def Volume(self, shift=0) -> float:
        '''
        Returns volume value for the default symbol with default timeframe and shift.

                Parameters:
                        shift (int): Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Volume price.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def Symbol(self) -> str:
        '''
        Returns the current symbol name.

                Returns:
                        The symbol name.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def GetOpenedOrderUIDs(self, symbol: str = None, scope: int = OrderScope.EA):
        '''
        Returns the UIDs of current opened orders.

                Parameters:
                        symbol: The symbol name.
                                If None returns current symbol.
                                If '*' returns all symbols.
                        scope:
                            EA: The current ea (default).
                            ACCOUNT: The current account.
                            EA_VERSION: The current ea version.

                Returns:
                        The uid list.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def GetPendingOrderUIDs(self, symbol: str = None, scope: int = OrderScope.EA):
        '''
        Returns the UIDs of current pending orders.

                Parameters:
                        symbol: The symbol name.
                                If None returns current symbol.
                                If '*' returns all symbols.
                        scope:
                            EA: The current ea (default).
                            ACCOUNT: The current account.
                            EA_VERSION: The current ea version.

                Returns:
                        The uid list.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def GetClosedOrderUIDs(self, symbol: str = None, scope: int = OrderScope.EA):
        '''
        Returns the UIDs of current closed orders.

                Parameters:
                        symbol: The symbol name.
                                If None returns current symbol.
                                If '*' returns all symbols.
                        scope:
                            EA: The current ea (default).
                            ACCOUNT: The current account.
                            EA_VERSION: The current ea version.

                Returns:
                        The uid list.
        '''
        raise NotImplementedError

    # --------------------------------
    @abc.abstractmethod
    def iMA(self, price_data, period, ma_type, shift=0):
        '''
        Calculates the Moving Average indicator and returns its value.

                Parameters:
                        price_data (object): The price data. (Close, Open, Low, etc...)
                        period (int): Averaging period for calculation.
                        ma_type: 0 (Simple Moving Average) ,For details see the TA-LIB
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Moving average value.
        '''
        raise NotImplementedError

    # https://mrjbq7.github.io/ta-lib/func_groups/volume_indicators.html
    @abc.abstractmethod
    def iAD(self, symbol_data, shift=0):
        '''
        Chaikin A/D Line (Volume Indicators)

                Parameters:
                        symbol_data (object): The symbol data.
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Chaikin A/D Line
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def iADX(self, symbol_data, timeperiod, shift=0):
        '''
        Average Directional Movement Index (Momentum Indicators)

                Parameters:
                        symbol_data (object): The symbol data.
                        timeperiod (int): The time period.
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Average Directional Movement Index
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def iATR(self, symbol_data, timeperiod, shift=0):
        '''
        Average True Range (Volatility Indicators)

                Parameters:
                        symbol_data (object): The symbol data.
                        timeperiod (int): The time period.
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Average True Range
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def iBands(self, symbol_data, timeperiod, nbdevup, nbdevdn, matype, shift=0):
        '''
        Bollinger Bands (Overlap Studies)

                Parameters:
                        symbol_data (object): The symbol data.
                        timeperiod (int): The time period.
                        nbdevup (int): 2
                        nbdevdn (int): 2
                        ma_type: 0 (Simple Moving Average) ,For details see the TA-LIB
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Bollinger Bands
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def iCCI(self, symbol_data, timeperiod, shift=0):
        '''
        Commodity Channel Index (Momentum Indicators)

                Parameters:
                        symbol_data (object): The symbol data.
                        timeperiod (int): The time period.
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Commodity Channel Index
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def iChaikin(self, symbol_data, fastperiod, slowperiod, shift=0):
        '''
         Chaikin A/D Oscillator (Volume Indicators)

                Parameters:
                        symbol_data (object): The symbol data.
                        fastperiod (int): The fast period.
                        slowperiod (int): The low period.
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Chaikin A/D Oscillator
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def iDEMA(self, symbol_data, timeperiod, shift=0):
        '''
        Double Exponential Moving Average (Overlap Studies)

                Parameters:
                        symbol_data (object): The symbol data.
                        timeperiod (int): The time period.
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Double Exponential Moving Average
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def iMomentum(self, symbol_data, timeperiod, shift=0):
        '''
        Momentum (Momentum Indicators)

                Parameters:
                        symbol_data (object): The symbol data.
                        timeperiod (int): The time period.
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Momentum
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def iMFI(self, symbol_data, timeperiod, shift=0):
        '''
        Money Flow Index (Momentum Indicators)

                Parameters:
                        symbol_data (object): The symbol data.
                        timeperiod (int): The time period.
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Money Flow Index
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def iMACD(self, symbol_data, fastperiod, slowperiod, signalperiod, shift=0):
        '''
        Moving Average Convergence/Divergence (Momentum Indicators)

                Parameters:
                        symbol_data (object): The symbol data.
                        fastperiod (int): The fast period.
                        slowperiod (int): The slow period.
                        signalperiod (int): The signal period.
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Moving Average Convergence/Divergence
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def iOBV(self, symbol_data, shift=0):
        '''
        On Balance Volume (Volume Indicators)

                Parameters:
                        symbol_data (object): The symbol data.
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        On Balance Volume
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def iSAR(self, symbol_data, acceleration, maximum, shift=0):
        '''
        Parabolic SAR (Overlap Studies)

                Parameters:
                        symbol_data (object): The symbol data.
                        acceleration (int): 0.02
                        maximum (int): 0.2
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Parabolic SAR
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def iRSI(self, symbol_data, timeperiod, shift=0):
        '''
        Relative Strength Index (Momentum Indicators)

                Parameters:
                        symbol_data (object): The symbol data.
                        timeperiod (int): The time period.
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Relative Strength Index
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def iStdDev(self, symbol_data, timeperiod, nbdev, shift=0):
        '''
        Standard Deviation (Statistic Functions)

                Parameters:
                        symbol_data (object): The symbol data.
                        timeperiod (int): The time period.
                        nbdev (int): 1
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Standard Deviation
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def iStochastic(self, symbol_data, fastk_period, slowk_period, slowk_matype, slowd_period, slowd_matype, shift=0):
        '''
        Stochastic (Momentum Indicators)

                Parameters:
                        symbol_data (object): The symbol data.
                        fastk_period (int): 5
                        slowk_period (int): 3
                        slowk_matype (int): 0
                        slowd_period (int): 3
                        slowd_matype (int): 0
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Stochastic
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def iTEMA(self, symbol_data, timeperiod, shift=0):
        '''
        Triple Exponential Moving Average (Overlap Studies)

                Parameters:
                        symbol_data (object): The symbol data.
                        timeperiod (int): The time period.
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Triple Exponential Moving Average
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def iWPR(self, symbol_data, timeperiod, shift=0):
        '''
        Williams' %R (Momentum Indicators)

                Parameters:
                        symbol_data (object): The symbol data.
                        timeperiod (int): The time period.
                        shift: Index of the value taken from the buffer
                        (shift relative to the current the given amount of periods ago).

                Returns:
                        Williams' %R
        '''
        raise NotImplementedError

    #
    @abc.abstractmethod
    def WaitCommand(self, uid, timeout=120) -> (ErrorID, CommandResult):
        '''
        Waiting for a asynchronous command execution。

                Parameters:
                        uid : The command UID.
                        timeout : Timeout （seconds）
                Returns:
                        ErrorID: If 0 success.
                        CommandResult: If failed returns None.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def AcquireLock(self, name, timeout=60) -> bool:
        '''
        Acquire a lock

                Parameters:
                        name : The lock name
                        timeout : Lock timeout （seconds）
                Returns:
                        If True success.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def ReleaseLock(self, name):
        '''
        Release a lock

                Parameters:
                        name : The lock name
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def Plot(self, series):
        '''
        Plot
                Parameters:
                        series
        '''
        raise NotImplementedError






from ..base.api_base import (API_V1_Base)
import numpy as np
import talib
import logging
from datetime import datetime
from pixiu.api.v1 import (TimeFrame, OrderCommand, OrderType, IndicatiorID, SymbolIndicator, SymbolPrice, SymbolData,
                      Order, Account, Symbol, AccountData, ErrorID, OrderResult, OrderScope)
log = logging.getLogger(__name__)


#---------------------------------------------------------------------------------------------------------------------
# TesterAPI_V1
#---------------------------------------------------------------------------------------------------------------------
class TesterAPI_V1(API_V1_Base):
    def __init__(self, tester, data_source, default_symbol):
        super(TesterAPI_V1, self).__init__(data_source, default_symbol)
        self.tester = tester

    def _print_(self, *args, **kwargs):
        """"""
        return self.tester.get_print_factory(*args, **kwargs)
    #
    def Buy(self, volume, type=OrderType.MARKET, price=0, stop_loss=None, take_profit=None, comment=None,
            magic_number=None, symbol=None, slippage=None, arrow_color=None, expiration=None):
        if symbol is None:
            symbol = self.default_symbol
        if price <= 0:
            price = self.Ask()

        if type == OrderType.MARKET:
            oct = OrderCommand.BUY
        elif type == OrderType.LIMIT:
            oct = OrderCommand.BUYLIMIT
        elif type == OrderType.STOP:
            oct = OrderCommand.BUYSTOP
        else:
            raise NotImplementedError

        return self.tester.open_order(symbol, oct, price, volume, stop_loss, take_profit,
                                           comment=comment, magic_number=magic_number, slippage=slippage,
                                           arrow_color=arrow_color)

    def Sell(self, volume, type=OrderType.MARKET, price=0, stop_loss=None, take_profit=None, comment=None,
             magic_number=None, symbol=None, slippage=None, arrow_color=None, expiration=None):
        if symbol is None:
            symbol = self.default_symbol
        if price <= 0:
            price = self.Bid()

        if type == OrderType.MARKET:
            oct = OrderCommand.SELL
        elif type == OrderType.LIMIT:
            oct = OrderCommand.SELLLIMIT
        elif type == OrderType.STOP:
            oct = OrderCommand.SELLSTOP
        else:
            raise NotImplementedError
        return self.tester.open_order(symbol, oct, price, volume, stop_loss, take_profit,
                                           comment=comment, magic_number=magic_number, slippage=slippage,
                                           arrow_color=arrow_color)

    def ModifyOrder(self, uid, price=None, stop_loss=None, take_profit=None, comment=None,
                     arrow_color=None, expiration=None):
        return self.tester.modify_order(uid, price, stop_loss, take_profit,
                              comment=comment, arrow_color=arrow_color, expiration=expiration)

    #
    def CloseOrder(self, uid, volume: float=None, price=None,  slippage=None, arrow_color=None):
        return self.tester.close_order(uid, volume, price, slippage=slippage,
                              arrow_color=arrow_color)

    def WaitCommand(self, uid, timeout=120):
        return self.tester.wait_command(uid, timeout)

    def AcquireLock(self, name, timeout=60):
        return self.tester.acquire_lock(name, timeout)

    def ReleaseLock(self, name):
        return self.tester.release_lock(name)

    def DefaultTimeFrame(self):
        return self.tester.tick_timeframe

    def GetSymbolData(self, symbol: str, timeframe: str, size: int):
        data = self.tester.get_data_info(symbol, timeframe)
        return SymbolData(data, timeframe, getitem_callback=self.tester.symbol_datagetitem_callback)

    def GetAccount(self):
        return Account(self.tester.get_account())

    def GetSymbol(self, symbol=None):
        if symbol is None:
            symbol = self.default_symbol
        sp = self.tester.get_symbol_properties(symbol)
        return Symbol(sp)

    def AccountEquity(self):
        return self.GetAccount().equity

    def AccountFreeMargin(self):
        return self.GetAccount().free_margin

    def SymbolInfo(self, item, symbol=None, default=None):
        if symbol is None:
            symbol = self.default_symbol
        sp = self.tester.get_symbol_properties(symbol)
        return sp.get(item, default)

    def OrderStats(self, order_uids):
        """"""
        ret = {}
        ret['count'] = len(order_uids)
        ret['volume'] = 0
        ret['profit'] = 0
        ret['open_highest'] = 0
        ret['open_lowest'] = 99999999
        #nHoldCount, nTotalVolume, nTotalProfit, nHightest, nLowest
        for oid in order_uids:
            order = self.GetOrder(oid)
            if order is None:
                continue
            ret['volume'] += order.volume
            ret['profit'] += order.profit
            if order.open_price > ret['open_highest']:
                ret['open_highest'] = order.open_price
            if order.open_price < ret['open_lowest']:
                ret['open_lowest'] = order.open_price
        return ret

    def Plot(self, series):
        self.tester.plot('default', series)

    def GetParam(self, name, default=None):
        '''Get params'''
        return self.get_param(name, default)

    def GetOrder(self, order_uid: str):
        return Order(self.tester.get_order(order_uid=order_uid))

    def Close(self, shift=0) -> float:
        return self.tester.Close(shift=shift)

    def Open(self, shift=0) -> float:
        return self.tester.Open(shift=shift)

    def High(self, shift=0) -> float:
        return self.tester.High(shift=shift)

    def Low(self, shift=0) -> float:
        return self.tester.Low(shift=shift)

    def Ask(self, shift=0) -> float:
        return self.tester.Ask(shift=shift)

    def Bid(self, shift=0) -> float:
        return self.tester.Bid(shift=shift)

    def Time(self, shift=0) -> datetime:
        return self.tester.Time(shift=shift)

    def Volume(self, shift=0) -> float:
        return self.tester.Volume(shift=shift)

    def Symbol(self, shift=0) -> str:
        return self.tester.symbol

    def GetOpenedOrderUIDs(self, symbol: str = None, scope: int = OrderScope.EA):
        ret = self.tester.get_order_dict(symbol, status="opened", scope=scope)
        return list(ret.keys())

    def GetPendingOrderUIDs(self, symbol: str = None, scope: int = OrderScope.EA):
        ret = self.tester.get_order_dict(symbol, status="pending", scope=scope)
        return list(ret.keys())

    def GetClosedOrderUIDs(self, symbol: str = None, scope: int = OrderScope.EA):
        ret = self.tester.get_order_dict(symbol, status="closed", scope=scope)
        return list(ret.keys())

    def StopTester(self, code: int=0, message: str=None):
        return self.tester.stop_tester(code, message)

    def __calculate_indicator__(self, indicator_id, price_data, *args, **kwargs):
        #indicator_id, price_data, period, ma_type
        if not isinstance(price_data, SymbolPrice) and not isinstance(price_data, SymbolData):
            return None
        if indicator_id == IndicatiorID.MA:
            ret = talib.MA(price_data.__price__, *args, **kwargs)
        elif indicator_id == IndicatiorID.AD:
            #high, low, close, volume
            ret = talib.AD(price_data.high.__price__, price_data.low.__price__, price_data.close.__price__,
                           price_data.volume.__price__)
        elif indicator_id == IndicatiorID.ADX:
            #high, low, close[, timeperiod=?]
            ret = talib.ADX(price_data.high.__price__, price_data.low.__price__, price_data.close.__price__,
                            *args, **kwargs)
        elif indicator_id == IndicatiorID.ATR:
            #(high, low, close[, timeperiod=?]
            ret = talib.ATR(price_data.high.__price__, price_data.low.__price__, price_data.close.__price__,
                            *args, **kwargs)
        elif indicator_id == IndicatiorID.BANDS:
            # real[, timeperiod=?, nbdevup=?, nbdevdn=?, matype=?]
            # nbDevUp
            # Deviation multiplier for upper band. Valid range from TRADER_REAL_MIN to TRADER_REAL_MAX.
            #
            # nbDevDn
            # Deviation multiplier for lower band. Valid range from TRADER_REAL_MIN to TRADER_REAL_MAX.
            upper, middle, lower = talib.BBANDS(price_data.__price__, *args, **kwargs)
            ret = np.array([upper, middle, lower]).T #or: np.transpose(a), a.transpose(
            # ret = []
            # for idx in range(upper.volume):
            #     ret.append((upper[idx], middle[idx], lower[idx]))
        elif indicator_id == IndicatiorID.CCI:
            # high, low, close[, timeperiod=?]
            ret = talib.CCI(price_data.high.__price__, price_data.low.__price__, price_data.close.__price__,
                            *args, **kwargs)
        elif indicator_id == IndicatiorID.CHAIKIN:
            # high, low, close, volume[, fastperiod=?, slowperiod=?]
            ret = talib.ADOSC(price_data.high.__price__, price_data.low.__price__, price_data.close.__price__,
                              price_data.volume.__price__, *args, **kwargs)
        elif indicator_id == IndicatiorID.DEMA:
            #DEMA(real[, timeperiod=?])
            ret = talib.DEMA(price_data.__price__, *args, **kwargs)
        elif indicator_id == IndicatiorID.MOMENTUM:
            # MOM(real[, timeperiod=?])
            ret = talib.MOM(price_data.__price__, *args, **kwargs)
        elif indicator_id == IndicatiorID.MFI:
            # MFI(high, low, close, volume[, timeperiod=?])
            ret = talib.MFI(price_data.high.__price__, price_data.low.__price__, price_data.close.__price__,
                              price_data.volume.__price__, *args, **kwargs)
        elif indicator_id == IndicatiorID.MACD:
            # MACD(real[, fastperiod=?, slowperiod=?, signalperiod=?])
            macd, macdsignal, macdhist = talib.MACD(price_data.__price__, *args, **kwargs)
            ret = np.array([macd, macdsignal, macdhist]).T #or: np.transpose(a), a.transpose(
        elif indicator_id == IndicatiorID.OBV:
            #  OBV(real, volume)
            ret = talib.OBV(price_data.__price__, *args, **kwargs)
        elif indicator_id == IndicatiorID.SAR:
            # SAR(high, low[, acceleration=?, maximum=?])
            ret = talib.SAR(price_data.high.__price__, price_data.low.__price__, *args, **kwargs)
        elif indicator_id == IndicatiorID.RSI:
            # RSI(real[, timeperiod=?])
            ret = talib.RSI(price_data.__price__, *args, **kwargs)
        elif indicator_id == IndicatiorID.STDDEV:
            # STDDEV(real[, timeperiod=?, nbdev=?])
            ret = talib.STDDEV(price_data.__price__, *args, **kwargs)
        elif indicator_id == IndicatiorID.STOCHASTIC:
            #  STOCH(high, low, close[, fastk_period=?, slowk_period=?, slowk_matype=?, slowd_period=?, slowd_matype=?])
            slowk, slowd = talib.STOCH(price_data.high.__price__, price_data.low.__price__, price_data.close.__price__,
                              *args, **kwargs)
            ret = np.array([slowk, slowd]).T #or: np.transpose(a), a.transpose(
        elif indicator_id == IndicatiorID.TEMA:
            # TEMA(real[, timeperiod=?])
            ret = talib.TEMA(price_data.__price__, *args, **kwargs)
        elif indicator_id == IndicatiorID.WPR:
            #  WILLR(high, low, close[, timeperiod=?])
            ret = talib.WILLR(price_data.high.__price__, price_data.low.__price__, price_data.close.__price__,
                              *args, **kwargs)
        else:
            return None

        return ret

    def __get_indicator__(self, cache_name, shift, indicator_id, price_data, *args, **kwargs):
        ret = price_data.indicators.get(cache_name, None)
        if ret is None:
            #data, ts_index, timeframe, getitem_callback
            ret = SymbolIndicator(self.__calculate_indicator__(indicator_id, price_data, *args, **kwargs),
                                  price_data.ts_index,
                                  price_data.timeframe,
                                  getitem_callback=price_data.__getitem_callback__)
            price_data.indicators[cache_name] = ret
        if ret is not None and shift is not None:
            return ret[shift]
        return ret

    def iMA(self, price_data, timeperiod, matype, shift=0):
        return self.__get_indicator__(f"ma:{matype}:{timeperiod}", shift, IndicatiorID.MA, price_data, timeperiod, matype)

    def iAD(self, symbol_data, shift=0):
        return self.__get_indicator__(f"ad:", shift, IndicatiorID.AD, symbol_data)

    def iADX(self, symbol_data, timeperiod, shift=0):
        '''
        Average Directional Movement Index
        Parameters
            symbol_data:
        '''
        # real = ADX(high, low, close, timeperiod=14)
        return self.__get_indicator__(f"adx:", shift, IndicatiorID.ADX, symbol_data, timeperiod)

    def iATR(self, symbol_data, timeperiod, shift=0):
        '''
        Average True Range
        Parameters
            symbol_data:
        '''
        # real = ADX(high, low, close, timeperiod=14)
        return self.__get_indicator__(f"atr:", shift, IndicatiorID.ATR, symbol_data, timeperiod)


    def iBands(self, symbol_data, timeperiod, nbdevup, nbdevdn, matype, shift=0):
        '''
        Bollinger Bands®
        Parameters
            symbol_data:
        '''
        # real = ADX(high, low, close, timeperiod=14)
        return self.__get_indicator__(f"bands:", shift, IndicatiorID.BANDS, symbol_data, timeperiod, nbdevup,
                                      nbdevdn, matype)


    def iCCI(self, symbol_data, timeperiod, shift=0):
        '''
        Commodity Channel Index
        Parameters
            symbol_data:
        '''
        # real = ADX(high, low, close, timeperiod=14)
        return self.__get_indicator__(f"cci:", shift, IndicatiorID.CCI, symbol_data, timeperiod)


    def iChaikin(self, symbol_data, fastperiod, slowperiod, shift=0):
        '''
        Chaikin Oscillator/ ADOSC                Chaikin A/D Oscillator
        Parameters
            symbol_data:
        '''
        # real = ADX(high, low, close, timeperiod=14)
        return self.__get_indicator__(f"chaikin:", shift, IndicatiorID.CHAIKIN, symbol_data, fastperiod, slowperiod)


    def iDEMA(self, symbol_data, timeperiod, shift=0):
        '''
        Double Exponential Moving Average
        Parameters
            symbol_data:
        '''
        # real = ADX(high, low, close, timeperiod=14)
        return self.__get_indicator__(f"dema:", shift, IndicatiorID.DEMA, symbol_data, timeperiod)


    def iMomentum(self, symbol_data, timeperiod, shift=0):
        '''
        Momentum / MOM                  Momentum
        Parameters
            symbol_data:
        '''
        # real = ADX(high, low, close, timeperiod=14)
        return self.__get_indicator__(f"momentum:", shift, IndicatiorID.MOMENTUM, symbol_data, timeperiod)


    def iMFI(self, symbol_data, timeperiod, shift=0):
        '''
        Money Flow Index
        Parameters
            symbol_data:
        '''
        # real = ADX(high, low, close, timeperiod=14)
        return self.__get_indicator__(f"mfi:", shift, IndicatiorID.MFI, symbol_data, timeperiod)


    def iMACD(self, symbol_data, fastperiod, slowperiod, signalperiod, shift=0):
        '''
        Moving Averages Convergence-Divergence
        Parameters
            symbol_data:
        '''
        # real = ADX(high, low, close, timeperiod=14)
        return self.__get_indicator__(f"macd:", shift, IndicatiorID.MACD, symbol_data, fastperiod, slowperiod,
                                      signalperiod)


    def iOBV(self, symbol_data, shift=0):
        '''
        On Balance Volume
        Parameters
            symbol_data:
        '''
        # real = ADX(high, low, close, timeperiod=14)
        return self.__get_indicator__(f"obv:", shift, IndicatiorID.OBV, symbol_data,
                                      symbol_data.__symbol_data__.volume.__price__)

    def iSAR(self, symbol_data, acceleration, maximum, shift=0):
        '''
        Parabolic Stop And Reverse System
        Parameters
            symbol_data:
        '''
        # real = ADX(high, low, close, timeperiod=14)
        return self.__get_indicator__(f"sar:", shift, IndicatiorID.SAR, symbol_data, acceleration, maximum)


    def iRSI(self, symbol_data, timeperiod, shift=0):
        '''
        Relative Strength Index
        Parameters
            symbol_data:
        '''
        # real = ADX(high, low, close, timeperiod=14)
        return self.__get_indicator__(f"rsi:", shift, IndicatiorID.RSI, symbol_data, timeperiod)

    def iStdDev(self, symbol_data, timeperiod, nbdev, shift=0):
        '''
        Standard Deviation
        Parameters
            symbol_data:
        '''
        return self.__get_indicator__(f"stddev:", shift, IndicatiorID.STDDEV, symbol_data, timeperiod, nbdev)

    def iStochastic(self, symbol_data, fastk_period, slowk_period, slowk_matype, slowd_period, slowd_matype, shift=0):
        '''
        Stochastic Oscillator
        Parameters
            symbol_data:
        '''
        # real = ADX(high, low, close, timeperiod=14)
        return self.__get_indicator__(f"stochastic:", shift, IndicatiorID.STOCHASTIC, symbol_data,
                                      fastk_period, slowk_period, slowk_matype, slowd_period, slowd_matype)

    def iTEMA(self, symbol_data, timeperiod, shift=0):
        '''
        Triple Exponential Moving Average
        Parameters
            symbol_data:
        '''
        # real = ADX(high, low, close, timeperiod=14)
        return self.__get_indicator__(f"tema:", shift, IndicatiorID.TEMA, symbol_data, timeperiod)

    def iWPR(self, symbol_data, timeperiod, shift=0):
        '''
        Williams' Percent Range
        Parameters
            symbol_data:
        '''
        # real = ADX(high, low, close, timeperiod=14)
        return self.__get_indicator__(f"wpr:", shift, IndicatiorID.WPR, symbol_data, timeperiod)




import math
import json
from datetime import (datetime, timedelta)
import random
import numpy as np
import pandas as pd
import time
from RestrictedPython.Guards import (guarded_unpack_sequence, )
from RestrictedPython.Eval import (default_guarded_getiter, )
import inspect
from pixiu.api.utils import (load_json, dump_json)

from ..api.v1 import (TimeFrame, OrderCommand, Order, APIStub as API_V1)

import logging
log = logging.getLogger(__name__)


class API_V1_Base(API_V1):
    """"""
    def __init__(self, data_source, default_symbol):
        self.data_source = data_source
        self.default_symbol = default_symbol

    def dict_to_order(self, order_dict):
        return Order(order_dict)

    # def datetime_(self, *args, **kwargs):
    #     return datetime(*args, **kwargs)

    #
    def default_guarded_getitem(self, ob, index):
        # No restrictions.
        return ob[index]

    def default_guarded_getattr(self, object, name, default=None):
        # No restrictions.
        return getattr(object, name, default)

    def default_unpack_sequence(self, *args, **kwargs):
        # No restrictions.
        #see: https://github.com/zopefoundation/RestrictedPython/blob/master/tests/transformer/test_assign.py
        return guarded_unpack_sequence(*args, **kwargs)
        # return getattr(object, name, default)

    def default_getiter(self, *args, **kwargs):
        # No restrictions.
        # return args.__iter__()
        return default_guarded_getiter(*args, **kwargs)

    def lock_object(self):
        return LockObject
    #
    def set_fun(self, env_dict):
        #BuildIn
        #see: https://github.com/zopefoundation/RestrictedPython/blob/9d3d403a97d7f030b6ed0f82900b2efac151dec3/tests/transformer/test_classdef.py
        env_dict["__metaclass__"] = type #support class
        env_dict["__name__"] = "ea_script" #support class
        env_dict["_write_"] = lambda x: x
        #support print
        env_dict["_print_"] = self._print_
        env_dict["_getitem_"] = self.default_guarded_getitem
        env_dict["_getiter_"] = self.default_getiter
        env_dict["_unpack_sequence_"] = self.default_unpack_sequence
        #System Functions
        env_dict["json"] = json
        env_dict["load_json"] = load_json
        env_dict["dump_json"] = dump_json
        env_dict["round"] = round
        env_dict["dict"] = dict
        env_dict["list"] = list
        env_dict['math'] = math
        env_dict['nan'] = np.nan
        env_dict['isnan'] = np.isnan
        env_dict['isnull'] = pd.isnull
        env_dict["datetime"] = datetime
        env_dict["timedelta"] = timedelta
        env_dict["random"] = random
        env_dict["time"] = time
        env_dict["pandas"] = pd
        env_dict["numpy"] = np
        #
        env_dict["LockObject"] = self.lock_object

        #
        members = inspect.getmembers(API_V1)
        for m in members:
            fun_n = m[0]
            if not fun_n.startswith('_'):
                env_dict[fun_n] = getattr(self, fun_n)
            elif fun_n == "__global_defines__":
                global_defines = getattr(self, fun_n)
                for k in global_defines:
                    env_dict[k] = global_defines[k]




from datetime import datetime

import numpy as np
from pixiu.api.utils import (parse_timeframe, calculate_minute_intervals, timeframe_to_seconds)


class Symbol(object):
    '''Symbol class'''
    def __init__(self, params={}):
        self.__symbol_dict__ = params.copy()

    @property
    def type(self) -> int:
        return int(self.__symbol_dict__["type"])

    @property
    def name(self) -> str:
        return str(self.__symbol_dict__["symbol"])

    @property
    def spread(self) -> int:
        return int(self.__symbol_dict__["spread"])

    @property
    def digits(self) -> int:
        return int(self.__symbol_dict__["digits"])

    @property
    def stop_level(self) -> int:
        return int(self.__symbol_dict__.get("stop_level", 0))

    @property
    def volume_min(self) -> float:
        return float(self.__symbol_dict__["volume_min"])

    @property
    def trade_contract_size(self) -> float:
        return float(self.__symbol_dict__["trade_contract_size"])

    @property
    def point(self) -> float:
        return float(self.__symbol_dict__["point"])

    @property
    def currency_profit(self) -> str:
        return str(self.__symbol_dict__["currency_profit"])

    @property
    def currency_base(self) -> str:
        return str(self.__symbol_dict__["currency_base"])

    @property
    def currency_margin(self) -> str:
        return str(self.__symbol_dict__["currency_margin"])


#---------------------------------------------------------------------------------------------------------------------
# SymbolData
#---------------------------------------------------------------------------------------------------------------------
class SymbolIndicator(object):
    def __init__(self, data, ts_index, timeframe, getitem_callback):
        self.__timeframe__ = timeframe
        self.__timeframe_seconds__ = timeframe_to_seconds(timeframe)
        self.data = data
        self.__ts_index__ = ts_index
        self.__getitem_callback__ = getitem_callback

    @property
    def ts_index(self):
        return self.__ts_index__

    @property
    def timeframe(self):
        return self.__timeframe__

    @property
    def timeframe_seconds(self):
        return self.__timeframe_seconds__

    def __getitem__(self, key):
        # return self.__getitem_callback__(self.data, self.__timeframe__, key)
        return self.__getitem_callback__(self.data, self.__ts_index__,
                                       self.__timeframe__,
                                       self.__timeframe_seconds__,
                                       key, fail_value=np.NaN)


class SymbolPrice(object):
    def __init__(self, symbol_data, price, getitem_callback):
        # self.__timeframe__ = timeframe
        # self.__timeframe_seconds__ = timeframe_to_seconds(timeframe)
        self.__price__ = price
        self.__symbol_data__ = symbol_data
        self.indicators = {}
        self.__getitem_callback__ = getitem_callback

    @property
    def ts_index(self):
        return self.__symbol_data__.ts_index

    @property
    def timeframe(self):
        return self.__symbol_data__.timeframe

    @property
    def timeframe_seconds(self):
        return self.__symbol_data__.timeframe_seconds

    def __getitem__(self, key):
        ret = self.__getitem_callback__(self.__price__, self.__symbol_data__.ts_index,
                                       self.__symbol_data__.timeframe,
                                       self.__symbol_data__.timeframe_seconds,
                                       key, fail_value=np.NaN)
        return ret


class SymbolTime(object):
    def __init__(self, symbol_data, price, getitem_callback):
        self.__time__ = price
        self.__symbol_data__ = symbol_data
        self.indicators = {}
        self.__getitem_callback__ = getitem_callback

    @property
    def ts_index(self):
        return self.__symbol_data__.ts_index

    @property
    def timeframe(self):
        return self.__symbol_data__.timeframe

    @property
    def timeframe_seconds(self):
        return self.__symbol_data__.timeframe_seconds

    def __getitem__(self, key):
        ts = self.__getitem_callback__(self.__time__, self.__symbol_data__.ts_index,
                                       self.__symbol_data__.timeframe,
                                       self.__symbol_data__.timeframe_seconds,
                                       key, fail_value=None)
        if ts is None:
            return None
        return datetime.fromtimestamp(ts)

class SymbolData(object):
    def __init__(self, data, timeframe, getitem_callback):
        self.__timeframe__ = timeframe
        self.__timeframe_seconds__ = timeframe_to_seconds(timeframe)
        self.__data__ = data
        self.__size__ = data.size
        self.__ts_index__ = data['t']
        self.__time__ = SymbolTime(self, data['t'], getitem_callback=getitem_callback)
        self.__close_price__ = SymbolPrice(self, data['c'], getitem_callback=getitem_callback)
        self.__open_price__ = SymbolPrice(self, data['o'], getitem_callback=getitem_callback)
        self.__high_price__ = SymbolPrice(self, data['h'], getitem_callback=getitem_callback)
        self.__low_price__ = SymbolPrice(self, data['l'], getitem_callback=getitem_callback)
        self.__ask_price__ = SymbolPrice(self, data['a'], getitem_callback=getitem_callback)
        self.__bid_price__ = SymbolPrice(self, data['b'], getitem_callback=getitem_callback)
        self.__volume__ = SymbolPrice(self, data['v'], getitem_callback=getitem_callback)
        self.indicators = {}
        self.__getitem_callback__ = getitem_callback

    def __getitem__(self, key):
        # return self.__getitem_callback__(self.__data__, key)
        return self.__getitem_callback__(self.__data__, self.__data__['t'],
                                       self.timeframe,
                                       self.timeframe_seconds,
                                       key)

    @property
    def timeframe(self):
        return self.__timeframe__

    @property
    def ts_index(self):
        return self.__ts_index__

    @property
    def timeframe_seconds(self):
        return self.__timeframe_seconds__

    @property
    def size(self):
        return self.__size__

    @property
    def close(self):
        return self.__close_price__

    @property
    def open(self):
        return self.__open_price__

    @property
    def high(self):
        return self.__high_price__

    @property
    def low(self):
        return self.__low_price__

    @property
    def ask(self):
        return self.__ask_price__

    @property
    def bid(self):
        return self.__bid_price__

    @property
    def time(self):
        return self.__time__

    @property
    def volume(self):
        return self.__volume__


from datetime import datetime

import numpy as np
from pixiu.api.defines import (OrderCommand)
from pixiu.api.utils import (parse_datetime_string)



class Order(object):

    def __init__(self, params={'volume': 0}):
        self.order_dict = params.copy()
        open_time = self.order_dict.get("open_time", None)
        if open_time is not None:
            if isinstance(open_time, int) or isinstance(open_time, float):
                self.order_dict["open_time"] = datetime.fromtimestamp(open_time)
            else:
                self.order_dict["open_time"] = parse_datetime_string(open_time)
        close_time = self.order_dict.get("close_time", None)
        if close_time is not None:
            if isinstance(close_time, int) or isinstance(close_time, float):
                self.order_dict["close_time"] = datetime.fromtimestamp(close_time)
            else:
                self.order_dict["close_time"] = parse_datetime_string(close_time)
        self.order_dict["commission"] = self.order_dict.get("commission", 0.0)
        self.order_dict["swap"] = self.order_dict.get("swap", 0.0)
        #cmd upper
        cmd = self.order_dict.get("cmd", None)
        if cmd is not None:
            self.order_dict["cmd"] = cmd.upper()

    def clone(self):
        return Order(self.order_dict.copy())

    def is_long(self) -> bool:
        long_types = [OrderCommand.BUY, OrderCommand.BUYLIMIT, OrderCommand.BUYSTOP]
        # cmd = self.cmd.upper()
        return self.cmd in long_types

    def is_short(self) -> bool:
        long_types = [OrderCommand.SELL, OrderCommand.SELLLIMIT, OrderCommand.SELLSTOP]
        # cmd = self.cmd.upper()
        return self.cmd in long_types

    def is_market(self) -> bool:
        market_types = [OrderCommand.BUY, OrderCommand.SELL]
        # cmd = self.cmd.upper()
        return self.cmd in market_types

    def is_stop(self) -> bool:
        stop_types = [OrderCommand.BUYSTOP, OrderCommand.SELLSTOP]
        # cmd = self.cmd.upper()
        return self.cmd in stop_types

    def is_limit(self) -> bool:
        limit_types = [OrderCommand.BUYLIMIT, OrderCommand.SELLLIMIT]
        # cmd = self.cmd.upper()
        return self.cmd in limit_types

    def is_pending(self) -> bool:
        return self.is_limit() or self.is_stop()

    @property
    def uid(self) -> str:
        return str(self.order_dict["uid"])

    @property
    def ticket(self) -> str:
        return str(self.order_dict.get("order_id", self.order_dict.get("ticket", None)))

    @property
    def profit(self) -> float:
        return float(self.order_dict.get("profit", 0))

    @property
    def margin(self) -> float:
        return float(self.order_dict.get("margin", None))

    @property
    def take_profit(self) -> float:
        value = self.order_dict.get("take_profit", None)
        if value is None:
            return None
        return float(value)

    @property
    def stop_loss(self) -> float:
        value = self.order_dict.get("stop_loss", None)
        if value is None:
            return None
        return float(value)

    @property
    def comment(self):
        return self.order_dict["comment"]

    @property
    def symbol(self):
        return str(self.order_dict["symbol"])

    @property
    def cmd(self):
        return str(self.order_dict["cmd"])

    @property
    def volume(self) -> float:
        return float(self.order_dict["volume"])

    @property
    def commission(self) -> float:
        return float(self.order_dict["commission"])

    @property
    def swap(self) -> float:
        return float(self.order_dict["swap"])

    @property
    def magic_number(self) -> float:
        ret = self.order_dict.get("magic_number", None)
        return int(ret) if ret is not None else None

    @property
    def open_price(self) -> float:
        return float(self.order_dict["open_price"])

    @property
    def open_time(self):
        return self.order_dict["open_time"]

    @property
    def close_time(self):
        return self.order_dict["close_time"]

    @property
    def close_price(self):
        return self.order_dict["close_price"] if self.order_dict["close_price"] is not None and not np.isnan(self.order_dict["close_price"]) else None

    @property
    def description(self):
        return self.order_dict["description"]





c_time = Time(valid_shift)

#default timeframe
timeframe = DefaultTimeFrame()
symbol_data = GetSymbolData(Symbol(), timeframe, 50)

#Test basic values
assertEqual(c_time, get_value_by_time(valid_values, c_time, "time"))
assertEqual(symbol_data.time[valid_shift], get_value_by_time(valid_values, c_time, "time"),
                f'@ {c_time}, {timeframe}')
assertEqual(symbol_data.open[valid_shift], get_value_by_time(valid_values, c_time, "open"),
                f'@ {c_time}, {timeframe}')
assertEqual(symbol_data.close[valid_shift], get_value_by_time(valid_values, c_time, "close"),
                f'@ {c_time}, {timeframe}')
assertEqual(symbol_data.high[valid_shift], get_value_by_time(valid_values, c_time, "high"),
                f'@ {c_time}, {timeframe}')
assertEqual(symbol_data.low[valid_shift], get_value_by_time(valid_values, c_time, "low"),
                f'@ {c_time}, {timeframe}')
assertEqual(symbol_data.ask[valid_shift], get_value_by_time(valid_values, c_time, "ask"),
                f'@ {c_time}, {timeframe}')
assertEqual(symbol_data.bid[valid_shift], get_value_by_time(valid_values, c_time, "bid"),
                f'@ {c_time}, {timeframe}')
assertEqual(symbol_data.volume[valid_shift], get_value_by_time(valid_values, c_time, "volume"),
                f'@ {c_time}, {timeframe}')

#default timeframe
timeframe_data = {TimeFrame.S1: {'seconds': 1}, TimeFrame.M1: {'seconds': 60}, TimeFrame.M5: {'seconds': 300},
                       TimeFrame.M15: {'seconds': 900}, TimeFrame.M30: {'seconds': 1800},
                       TimeFrame.H1: {'seconds': 3600}, TimeFrame.H4: {'seconds': 14400},
                       TimeFrame.D1: {'seconds': 86400}}
for timeframe in timeframe_data:
    symbol_data = GetSymbolData(Symbol(), timeframe, 50)

    #Test basic values
    assertEqual(symbol_data.time[valid_shift], get_timeframe_value_by_time(timeframe, c_time, "time"),
                f'@ {c_time}, {timeframe}')
    assertEqual(symbol_data.open[valid_shift], get_timeframe_value_by_time(timeframe, c_time, "open"),
                f'@ {c_time}, {timeframe}')
    assertEqual(symbol_data.close[valid_shift], get_timeframe_value_by_time(timeframe, c_time, "close"),
                f'@ {c_time}, {timeframe}')
    assertEqual(symbol_data.high[valid_shift], get_timeframe_value_by_time(timeframe, c_time, "high"),
                f'@ {c_time}, {timeframe}')
    assertEqual(symbol_data.low[valid_shift], get_timeframe_value_by_time(timeframe, c_time, "low"),
                f'@ {c_time}, {timeframe}')
    assertEqual(symbol_data.ask[valid_shift], get_timeframe_value_by_time(timeframe, c_time, "ask"),
                f'@ {c_time}, {timeframe}')
    assertEqual(symbol_data.bid[valid_shift], get_timeframe_value_by_time(timeframe, c_time, "bid"),
                f'@ {c_time}, {timeframe}')
    assertEqual(symbol_data.volume[valid_shift], get_timeframe_value_by_time(timeframe, c_time, "volume"),
                f'@ {c_time}, {timeframe}')

set_test_result("OK")
#
# StopTester()


from .api import *
from .order import (Order)
from .account import (Account, AccountDataNumber, AccountDataString, AccountDataTime, AccountData)
from .symbol import (Symbol, SymbolIndicator, SymbolPrice, SymbolTime, SymbolData)
